# Generated by sila2.code_generator; sila2.__version__: 0.12.2
from os.path import dirname, join

from sila2.framework import Feature

SessionManagementFeature = Feature(join(dirname(__file__), "SessionManagement.sila.xml"))



# Generated by sila2.code_generator; sila2.__version__: 0.12.2
from __future__ import annotations

import string
from typing import TYPE_CHECKING

from sila2.server import MetadataDict

from ..generated.sessionmanagement import (
    SessionCreate_Responses,
    SessionDelete_Responses,
    SessionManagementBase,
)

if TYPE_CHECKING:
    from ..server import Server

import opentrons.execute
import opentrons.simulate
import uuid

from ..base import Base
from ..labware.labware_wrapper import LabwareWrapper


class SessionManagementImpl(SessionManagementBase):
    def __init__(self, parent_server: Server, base: Base, simulate: bool=False) -> None:
        super().__init__(parent_server=parent_server)
        self.base = base
        self.should_simulate = simulate

    def SessionCreate(self, *,
                      metadata: MetadataDict) -> SessionCreate_Responses:
        new_id = self.base.start_session()
        if new_id != "":
            if self.should_simulate:
                protocol = opentrons.simulate.get_protocol_api("2.19", robot_type="Flex")
            else:
                protocol = opentrons.execute.get_protocol_api("2.19")
            tiprack = protocol.load_labware(
                load_name="opentrons_flex_96_tiprack_1000ul", location="A2"
            )
            # attach pipette to left mount
            pipette = protocol.load_instrument(
                instrument_name="flex_8channel_1000",
                mount="left",
                tip_racks=[tiprack]
            )
            trash = protocol.load_trash_bin(location="A3")
            self.base.set_instrument(pipette)
            self.base.set_session_protocol(protocol)
            self.base.add_labware(LabwareWrapper(tiprack, "A2"),
                                  str(uuid.uuid4()))
            self.base.set_trash(trash)
            return SessionCreate_Responses(SessionToken=new_id)
        raise Exception("Machine is already in use")

    def SessionDelete(self, SessionToken: str, *,
                      metadata: MetadataDict) -> SessionDelete_Responses:
        self.base.end_session(SessionToken)
        return SessionDelete_Responses()



# Generated by sila2.code_generator; sila2.__version__: 0.12.2
from __future__ import annotations

from abc import ABC
from sila2.server import FeatureImplementationBase

from abc import abstractmethod



from sila2.server import MetadataDict








from .labware_types import RegisterLabware_Responses

from .labware_types import DeleteLabware_Responses

from .labware_types import MoveLabware_Responses



from typing import TYPE_CHECKING
if TYPE_CHECKING:

    from ...server import Server


class LabwareBase(FeatureImplementationBase, ABC):
    parent_server: Server





    def __init__(self, parent_server: Server):
        """
        This feature allows you to manipulate the labware in the OT-2 and
        OT-Flex
        """
        super().__init__(parent_server=parent_server)

    

    




    @abstractmethod
    def RegisterLabware(
        self,
        SessionIdentifier: str,LoadName: str,Location: str,
        *,
        metadata: MetadataDict
    ) -> RegisterLabware_Responses:
        """
        This method registers labware to a specified position for
            the current session

    
        :param SessionIdentifier: UUID identifier for the current session
    
        :param LoadName: The name of the standard type of labware to register
                (https://labware.opentrons.com)
    
        :param Location: Location for labware, modules, adapters, etc.
    
        :param metadata: The SiLA Client Metadata attached to the call
    
        :return:
        
            - ItemIdentifier: UUID identifier for referenceable labware, modules,
                adapters, etc.
        
    
        """

    @abstractmethod
    def DeleteLabware(
        self,
        SessionIdentifier: str,Identifier: str,
        *,
        metadata: MetadataDict
    ) -> DeleteLabware_Responses:
        """
        This method deletes labware without removing it, Only use this
            method if you are removing it yourself and make sure to not execute
            any commands while moving it.

    
        :param SessionIdentifier: UUID identifier for the current session
    
        :param Identifier: Used to determine what labware to remove. This has to
                be an Item Identifier or a Location in the format used in other
                commands
    
        :param metadata: The SiLA Client Metadata attached to the call
    
        """

    @abstractmethod
    def MoveLabware(
        self,
        SessionIdentifier: str,ItemIdentifier: str,Location: str,
        *,
        metadata: MetadataDict
    ) -> MoveLabware_Responses:
        """
        This method deletes labware without removing it, Only use this
            method if you are removing it yourself and make sure to not execute
            any commands while moving it.

    
        :param SessionIdentifier: UUID identifier for the current session
    
        :param ItemIdentifier: UUID identifier for referencable labware, modules,
                adapters, etc.
    
        :param Location: Location for labware, modules, adapters, etc.
    
        :param metadata: The SiLA Client Metadata attached to the call
    
        """








from opentrons.protocols.models.json_protocol import Labware


class LabwareWrapper:
    def __init__(self, labware: Labware, position: str) -> None:
        self.labware = labware
        self.position = position
